@@ -3,6 +3,7 @@ use warnings;
 
 package Net::IMP::Base;
 use base 'Net::IMP';
+use Carp 'croak';
 use fields (
     'meta', # hash with meta data given in new_analyzer
     'cb',   # callback, set from new_analyzer or with set_callback
@@ -49,6 +50,33 @@ sub run_callback {
 }
 
 
+# make string from hash config, using URL encoding to escape special chars
+sub cfg2str {
+    my Net::IMP::Base $self = shift;
+    my $cfg = shift;
+    return join('&', map {
+	my $v = $cfg->{$_};
+	# only encode really necessary stuff
+	s{([=&%\x00-\x20\x7f-\xff])}{ sprintf("%%02X",ord($1)) }eg; # key
+	s{([&%\x00-\x20\x7f-\xff])}{ sprintf("%%02X",ord($1)) }eg;  # value
+	"$_=$v"
+    } sort keys %$cfg);
+}
+
+# make has config from string created by cfg2str
+sub str2cfg {
+    my Net::IMP::Base $self = shift;
+    my $str = shift;
+    my %cfg;
+    for my $kv (split('&',$str)) {
+	my ($k,$v) = split('=',$kv,2);
+	s{%([\dA-F][\dA-F]])}{ chr(hex($1)) }ieg for ($k,$v);
+	exists $cfg{$k} and croak "duplicate definition for key $k";
+	$cfg{$k} = $v;
+    }
+    return \%cfg;
+}
+
 1;
 __END__
 
@@ -181,3 +209,20 @@ results.
 It will only be used from the caller of the analyzer if no callback is set.
 
 =back
+
+Additionally the following methods are defined to aid in using configuration
+from a single string:
+
+=over 4
+
+=item $class->cfg2str(\%config) -> $string
+
+Creates a string from a (configuration) hash.
+The output is similar to encoded query parameters in URLs.
+
+=item $class->str2cfg($string) -> \%config
+
+Parses a configuration string generated by C<cfg2str> and restores the
+configuration hash.
+
+=back
